import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";

export const get = async (request: Request, response: Response) => {
    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const feed = await prisma.feed.findUnique({
        where: { id: Number(request.query.id) },
        include: { channels: true }
    });

    if (!feed) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(feed);
};

export const post = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const { name, url, active, channels } = request.body;

    let feed;

    try {
        feed = await prisma.feed.create({
            data: { 
                name: name,
                url: url,
                guild_id: guildId,
                active: active,
                channels: channels
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(feed);
};

export const patch = async () => {}  // TODO ...

export const del = async (request: Request, response: Response) => {
    const { ids } = request.body;
    const guildId = request.params.guildId;

    if (!ids || !Array.isArray(ids)) {
        response.status(400).json({ error: "invalid request body" });
        return;
    }

    try {
        await prisma.feed.deleteMany({ where: {
            id: { in: ids },
            guild_id: guildId
        }});
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(204).json(null);
};

interface DataTableResponse {
    data: any;
    recordsFiltered: number;
    recordsTotal: number;
}

interface DatatableQuery {
    length: string;
    start: string;
    order: { column: string; dir: string }[];
    columns: { [key: string]: { data: string; searchable: string }};
    search: { value: string };
    filters: { [key: string]: any };
}

export const datatable = async (request: Request, response: Response) => {
    const query = request.query as unknown as DatatableQuery;

    const size: number = Number(query.length) || 10;
    const start: number = Number(query.start);
    const order: string = (query.order && query.columns[query.order[0].column].data) || "id";
    const direction: string = (query.order && query.order[0].dir) || "asc";
    const search: string = query.search?.value || "";

    let dbQuery: any = {};

    // TODO: filter request

    if (search) {
        Object.values(query.columns)
            .filter(column => column.searchable === "true")
            .forEach((col: any) => {
                dbQuery["where"][col.data] = {
                    contains: search,
                    mode: "insensitive"
                }
            }
        );
    }

    const orderBy: any = {};
    orderBy[order] = direction;

    const data = await prisma.feed.findMany({
        ...dbQuery,
        skip: start,
        take: size,
        orderBy: orderBy,
        include: { channels: true }
    });

    response.json(<DataTableResponse>{
        data: data,
        recordsFiltered: await prisma.feed.count({...dbQuery}),
        recordsTotal: await prisma.feed.count()
    });
};

export default { get, post, patch, del, datatable };